#!/usr/bin/env python2.5

#
# Copyright (c) 2007-2008 INdT.
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# ============================================================================
# Name        : calllog.py
# Author      : kovach
# Version     : 0.1
# Description : Detailed Call Log
# ============================================================================

# If you can't run this, try running this command first:
# apt-get install python-gtk2 python-gobject python-hildon python-dbus gnome-python

import gtk
import gtk.gdk
import gobject
import hildon
import time
import sqlite3
import dbus
from gnome import gconf

gobject.threads_init()

EL_PATH = "/home/user/.rtcom-eventlogger/el.db"
IMG_DIALED = "/usr/share/icons/hicolor/24x24/hildon/general_sent.png"
IMG_RECIEVED = "/usr/share/icons/hicolor/24x24/hildon/general_received.png"
IMG_MISSED = "/usr/share/icons/hicolor/24x24/hildon/general_missed.png"

class PyCallLogApp(hildon.Program):
    def __init__(self):
        hildon.Program.__init__(self)

        application = 'Call Log'
        gtk.set_application_name(application)

        self.main_window = hildon.Window()
        self.main_window.set_title(application)
        self.main_window.connect("delete_event", self.quit)
        self.add_window(self.main_window)

        self.menu = hildon.AppMenu()
        button1 = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
        button1.set_label("Settings")
        button1.connect("clicked", self.do_settings)
        self.menu.append(button1)
        button2 = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
        button2.set_label("Refresh")
        button2.connect("clicked", self.reload_data)
        self.menu.append(button2)
        self.menu.show_all()
        self.main_window.set_app_menu(self.menu)

        self.pan_area = hildon.PannableArea()
        self.main_window.add(self.pan_area)

        self.treeview = gtk.TreeView()
        self.pan_area.add(self.treeview)

        cell = gtk.CellRendererPixbuf()
        cell.set_property('xalign', 1.0)
        tvcolumn = gtk.TreeViewColumn('Type', cell, pixbuf = 0)
        self.treeview.append_column(tvcolumn)

        tvcolumn = gtk.TreeViewColumn('Time', gtk.CellRendererText(), text = 1)
        self.treeview.append_column(tvcolumn)

        tvcolumn = gtk.TreeViewColumn('Number', gtk.CellRendererText(), text = 2)
        self.treeview.append_column(tvcolumn)

        tvcolumn = gtk.TreeViewColumn('Name', gtk.CellRendererText(), text = 3)
        self.treeview.append_column(tvcolumn)

        self.treeview.set_search_column(3)
        self.treeview.connect('row-activated', self.item_activated)

        self.reload_data()

        self.pan_area.show()


    def reload_data(self, event = None, param = None):
        conf_client = gconf.client_get_default()
        if not conf_client.dir_exists ('/apps/pycalllog/general'):
            conf_client.add_dir ("/apps/pycalllog/general", gconf.CLIENT_PRELOAD_NONE)
        daysToShow = conf_client.get_int("/apps/pycalllog/general/daysToShow")
        if daysToShow == None or not isinstance(daysToShow, int) or daysToShow == 0:
            daysToShow = 30
            conf_client.set_int("/apps/pycalllog/general/daysToShow", daysToShow)

        starttime = time.mktime(time.localtime()) - (daysToShow * 24 * 60 * 60)

        conn = sqlite3.connect(EL_PATH)
        cur = conn.cursor()
        cur.execute("select event_type_id, start_time, E.remote_uid, remote_name from Events E, Remotes R " +
                    "where R.remote_uid=E.remote_uid and E.local_uid=R.local_uid and service_id = 1 " +
                    "and start_time > " + str(starttime) + " order by start_time desc")

        pix_dialed = gtk.gdk.pixbuf_new_from_file(IMG_DIALED)
        pix_recieved = gtk.gdk.pixbuf_new_from_file(IMG_RECIEVED)
        pix_missed = gtk.gdk.pixbuf_new_from_file(IMG_MISSED)

        treestore = gtk.ListStore(gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)

        for datarow in cur:
            if (datarow[0] == 1):
                type = pix_recieved
            elif (datarow[0] == 2):
                type = pix_dialed
            else:
                type = pix_missed

            thetime = time.strftime("%a, %d %b %H:%M:%S", time.localtime(datarow[1]))
            treestore.append(row = [type, thetime, datarow[2], datarow[3]])

        self.treeview.set_model(treestore)


    def do_settings(self, event = None, param = None):
        conf_client = gconf.client_get_default()
        daysToShow = conf_client.get_int("/apps/pycalllog/general/daysToShow")
        if daysToShow == None or not isinstance(daysToShow, int) or daysToShow == 0:
            daysToShow = 30
            conf_client.set_int("/apps/pycalllog/general/daysToShow", daysToShow)

        dialog = gtk.Dialog('Settings', self.main_window)
        dialog.add_button('Save', gtk.RESPONSE_OK)

        content = dialog.vbox

        label = gtk.Label('Number of days to show:')
        label.set_justify(gtk.JUSTIFY_LEFT)
        content.add(label)

        tb_days = hildon.Entry(gtk.HILDON_SIZE_FINGER_HEIGHT)
        tb_days.set_property('is-focus', True)
        tb_days.set_property('hildon-input-mode', gtk.HILDON_GTK_INPUT_MODE_FULL)
        tb_days.set_text(str(daysToShow))
        content.add(tb_days)

        dialog.show_all()
        result = dialog.run()
        dialog.hide()
        if result == gtk.RESPONSE_OK:
            daysToShow = int(tb_days.get_text())
            conf_client.set_int("/apps/pycalllog/general/daysToShow", daysToShow)
            self.reload_data()

        return result


    def item_activated(self, treeview, path, column):
        iter    = treeview.get_model().get_iter(path)
        number = treeview.get_model().get_value(iter, 2)
        self.place_call(number)

    def place_call(self, number):
        #dbus-send --system --dest=com.nokia.csd.Call --type=method_call --print-reply /com/nokia/csd/call com.nokia.csd.Call.CreateWith string:"$1" uint32:0
        bus = dbus.SystemBus()
        csd_call = dbus.Interface(bus.get_object('com.nokia.csd',
                                                '/com/nokia/csd/call'),
                                                'com.nokia.csd.Call')
        csd_call.CreateWith(str(number), dbus.UInt32(0))

    def quit(self, *args):
        gtk.main_quit()

    def run(self):
        self.main_window.show_all()
        gtk.main()


if __name__ == "__main__":
    app = PyCallLogApp()
    app.run()
